//;--------------------------------------------------------------------------------------------------------------
//; ws2811_hs.S - ws2811 high speed write routine for rgb controller
//;--------------------------------------------------------------------------------------------------------------
//;
//;    Copyright (C) 2012  Kevin Timmerman
//;
//;   This program is free software: you can redistribute it and/or modify
//;   it under the terms of the GNU General Public License as published by
//;   the Free Software Foundation, either version 3 of the License, or
//;   (at your option) any later version.
//;
//;   This program is distributed in the hope that it will be useful,
//;   but WITHOUT ANY WARRANTY//; without even the implied warranty of
//;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//;   GNU General Public License for more details.
//;
//;   You should have received a copy of the GNU General Public License
//;   along with this program.  If not, see <http://www.gnu.org/licenses/>.
//;--------------------------------------------------------------------------------------------------------------
//; 11-15-2012 - rick@kimballsoftware.com modified for msp430-gcc
//;--------------------------------------------------------------------------------------------------------------

#include "msp430.h"
#include "driverlib.h"

// data - R12, length - R13, pinmask - R14, portmask - 0(R15)
//
// NOTE: assumes interrupts are disabled as cycle counts are used for timing
//
// for 25MHz (using new datasheet)
// Zero:  0.4 / 0.85         10 / 21.25
// One:   0.8 / 0.45         20 / 11.25
// Reset:    0 / 50+         0 / 1250
//
//void write_ws2811_25(uint8_t *data, uint16_t length)
void write_ws2811_25(uint8_t *data, uint16_t length, uint8_t pinmask, uint16_t portmask)
{
asm("                                ; number in [] is # of cycles");
asm("write_ws2811_25_asm:");
asm("        push    R10             ; [5] Save R10");
asm("        push    R11             ; [5] Save R11");
asm("                                ;");
asm("byte_loop_hs:                   ;          --- continue for T0L/T1L when starting next byte");
asm("        mov     #7, R10         ;          [2] Do 7 bits in a loop");
asm("        mov.b   @R12+, R11      ;          [2] Get next byte from buffer");
asm("                                ;          --- so far 18/8 cycles for T0L/T1L, 3 left");
asm("                                ;");
asm("bit_loop_hs:                    ;     --- continue from T0L/T1L");
asm("        rla.b   R11             ;     [1] Get next bit");
asm("        jc      one_hs          ;     [2] Jump if sending 1...");
asm("                                ;     --- 21/11 cycles for T0L/T1L");
asm("                                ;");
asm("                                ; --- start of T0H");
asm("        bic.b   R14, 0(R15)     ; [4] Output High");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("                                ; --- 10 cycles for T0H");
asm("                                ;");
asm("                                ; --- start of T0L");
asm("        bis.b   R14, 0(R15)     ; [4] Output Low");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        jmp     next_bit_hs     ; [2] Next bit...");
asm("                                ; --- so far 14 cycles for T0L, 7 left");
asm("                                ;");
asm("one_hs:                         ; --- start of T1H");
asm("        bic.b   R14, 0(R15)     ; [4] Output High");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("                                ; --- 20 cycles for T1H");
asm("                                ;");
asm("                                ; --- start of T1L");
asm("        bis.b   R14, 0(R15)     ; [4] Output Low");
asm("                                ; --- so far 4 cycles for T1L, 7 left");
asm("                                ;");
asm("next_bit_hs:                    ; --- continue for T0L/T1L");
asm("        dec     R10             ; [2] Decrement bit count");
asm("        jne     bit_loop_hs     ; [2] Do next bit if not last bit...");
asm("                                ; --- so far 18/8 cycles for T0L/T1L, 3 left");
asm("                                ;");
asm("        rla.b   R11             ; [1] Get final bit of byte");
asm("        jc      last_one_hs     ; [2] Jump if send 1...");
asm("                                ; --- 21/11 cycles for T0L/T1L");
asm("                                ;");
asm("                                ; --- start of T0H");
asm("        bic.b   R14, 0(R15)     ; [4] Output High");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("                                ; --- 10 cycles for T0H");
asm("                                ;");
asm("                                ; --- start of T0L");
asm("        bis.b   R14, 0(R15)     ; [4] Output Low");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        dec     R13             ; [2] Decrement byte count");
asm("        jne     byte_loop_hs    ; [2] Next byte if count not zero...");
asm("                                ; --- so far 14 cycles for T0L, 7 left");
asm("        jmp     reset_hs        ; [2] All bytes done, reset...");
asm("                                ;");
asm("last_one_hs:                    ; --- start of T1H");
asm("        bic.b   R14, 0(R15)     ; [4] Output High");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        nop                     ; [1]");
asm("        mov     #7, R10         ; [2] Do 7 bits in a loop (prep because T1L is too short)");
asm("        mov.b   @R12+, R11      ; [2] Get next byte from buffer (prep because T1L is too short)");
asm("                                ;          --- so far 18/8 cycles for T0L/T1L, 3 left");
asm("                                ; --- 20 cycles for T1H");
asm("                                ;");
asm("                                ; --- start of T1L");
asm("        bis.b   R14, 0(R15)     ; [4] Output Low");
asm("        dec     R13             ; [2] Decrement byte count");
asm("        jne     bit_loop_hs     ; [2] Do next byte if count is not zero...");
asm("                                ; --- so far 8 cycles for T1L, 3 left");
asm("                                ;");
asm("reset_hs:                       ; --- start reset");
asm("        mov     #1250 / 3, R11  ; [2] 800 cycle delay for reset");
asm("        dec     R11             ; [2 * 416] this is run (1250/3) or 416 times for 832 cycles");
asm("        jne     $ - 2           ; [2 * 416] this is run (1250/3) or 416 times for 832 cycles");
asm("                                ; --- 1666 cycles");
asm("        pop     R11             ; [5?] Restore R10");
asm("        pop     R10             ; [5?] Restore R11");
}
