﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace dashboard2015
{
    class MeterControl : Panel
    {
        private bool _leftToRight = true;
        private bool _horizontalLayout = true;
        private int _value = 0;
        private const int _maxValue = 100;

        public bool LeftToRight
        {
            get { return _leftToRight; }
            set { _leftToRight = value; }
        }

        public bool HorizontalLayout
        {
            get { return _horizontalLayout; }
            set { _horizontalLayout = value; }
        }

        public int Value
        {
            get { return _value; }
            set
            {
                // If the value has changed
                if (_value != value)
                {
                    // The value must be between 0 and 100. 
                    if (value <= _maxValue && value >= 0)
                    {
                        _value = value;
                        // This repaints the control. Per MSDN: Calling the Invalidate method does not force a synchronous paint;
                        // to force a synchronous paint, call the Update method after calling the Invalidate method.
                        this.Invalidate();
                        this.Update();
                    }
                }
            }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            DrawFillRectangle();
        }

        private void DrawFillRectangle()
        {
            // Draws the green rectangle indicating the value set.
            System.Drawing.SolidBrush myBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Lime);
            System.Drawing.Graphics formGraphics;
            formGraphics = this.CreateGraphics();

            int width = 0;
            int height = 0;
            int x = 0;
            int y = 0;

            double percent = (double)_value / (double)_maxValue;
            if (HorizontalLayout)
            {
                height = this.Height;
                // Width should be a percentage of control 
                width = (int)(this.Width * percent);

                if (!LeftToRight)
                {
                    x = this.Width - width; // Need to change the x position of the rectangle to be drawn
                }
            }
            else // vertical layout
            {
                width = this.Width;
                // Height should be a percentage of control;
                height = (int)(this.Height * percent);

                y = this.Height - height;
            }

            formGraphics.FillRectangle(myBrush, new Rectangle(x, y, width, height));
            myBrush.Dispose();

            StringFormat f = new StringFormat();
            f.LineAlignment = StringAlignment.Center;
            f.Alignment = StringAlignment.Center;


            formGraphics.DrawString(((int)(100.0*percent)).ToString(), new Font("Arial", 18), Brushes.Black, this.ClientRectangle, f);
            formGraphics.Dispose();
        }
    }
}
