#include "action.h"

Action::Action() {
}

Action::~Action() {
}

void Action::postHook() {

}
void Action::preHook() {

}

SequentialGroup::SequentialGroup() {
	index = 0;
}
SequentialGroup::~SequentialGroup() {
	for (Action* a : actions) {
		delete a;
	}
}
SequentialGroup::SequentialGroup(std::initializer_list<Action*> l) {
	index = 0;
	actions = l;
}
void SequentialGroup::add(Action* a) {
	actions.push_back(a);
}
void SequentialGroup::initialize() {
	index = 0;
	if (index < actions.size()) {
		actions[index]->preHook();
		actions[index]->initialize();
	}
}
void SequentialGroup::process() {
	if (index < actions.size()) {
		actions[index]->process();
		if (actions[index]->isFinished()) {
			actions[index]->postHook();
			index++;
			if (index < actions.size()) {
				actions[index]->preHook();
				actions[index]->initialize();
			}
		}
	}
}
void SequentialGroup::abort() {
	if (index < actions.size()) {
		actions[index]->abort();
		actions[index]->postHook();
	}
}
bool SequentialGroup::isFinished() {
	return index == actions.size();
}

ParallelGroup::ParallelGroup() {
}
ParallelGroup::~ParallelGroup() {
	for (Combo& c : actions) {
		delete c.action;
	}
}
ParallelGroup::ParallelGroup(std::initializer_list<Action*> l) {
	for (Action* a : l) {
		add(a);
	}
}
void ParallelGroup::add(Action* a) {
	actions.push_back( { a, false });
}
void ParallelGroup::initialize() {
	for (Combo& c : actions) {
		c.action->initialize();
		c.action->preHook();
	}
}
void ParallelGroup::process() {
	for (Combo& c : actions) {
		if (!c.done) {
			c.action->process();
			c.done = c.action->isFinished();
			if (c.done) {
				c.action->postHook();
			}

		}
	}
}
void ParallelGroup::abort() {
	for (Combo& c : actions) {
		if (!c.done) {
			c.action->abort();
			c.action->postHook();
		}
	}
}
bool ParallelGroup::isFinished() {
	for (Combo& c : actions) {
		if (!c.done) {
			return false;
		}
	}
	return true;
}
