#include "closedloop.h"
#include <util/common.h>
#include "util/misc.h"

#define DEBUG true

PIDFA::PIDFA(real iP, real iI, real iD, real iF, real iA, real iContribCap) :
				P(iP),
				I(iI),
				D(iD),
				F(iF),
				A(iA),
				iCap(iContribCap / iA) {
	Reset();
}
PIDFA::~PIDFA() {

}
void PIDFA::Reset() {
	lastTime = GetTime();
	lastErr = 0;
	integral = 0;
}

real PIDFA::getNextOutput(real setX, real X, real goalD) {
	real err = setX - X;

	double time = GetTime();
	real step = time - lastTime;
	lastTime = time;

	integral += err * step;
	if (iCap != 0) {
		integral = bound(integral, -iCap, iCap);
	}

	real diff = (err - lastErr) / step;
	lastErr = err;

	real output = P * lastErr + I * integral + D * diff + F * setX + A * goalD;
#if DEBUG
	printf("G %.3f <> R %.3f = %.3f -> %.3f %.3f %.3f %.3f %.3f\n", setX, X,
			output, P * lastErr, I * integral, D * diff, F * setX, A * goalD);
#endif
	return output;
}
