#pragma once

#include <util/sensors.h>
#include "PowerDistributionPanel.h"
#include "Solenoid.h"
#include "Encoder.h"
#include <ctre/CanTalonSRX.h>
#include "AnalogInput.h"
#include "DigitalInput.h"
#include "ADXL345_SPI.h"
#include "BuiltInAccelerometer.h"

/////////////////////////////////////

// Comment this out on robot 1
#define ROBOT_2

/////////////////////////////////////

#define ONE_CHANNEL_DRIVE true

typedef struct CANTalonRates_struct {
	uint8_t general;
	uint8_t analogTempVbat;
	uint8_t encoder;
	uint8_t feedback;
} CANTalonRates;

class CachedTalon : private CanTalonSRX {
public:
	CachedTalon(uint32_t id);
	void updateCurrent();
	double getLastCurrent();
	void setUpdateRates(CANTalonRates r);
	void SetExpectedCommand(double setpoint);
	void ActuallyDoExpectedCommand();
private:
	double lastCurrent;
	double expComm;
	double lastComm;
};

class CachedSolenoid : private Solenoid {
public:
	CachedSolenoid(uint8_t moduleNumber, uint32_t channel);
	void SetOnlyIfChanged(bool on);
private:
	bool lastState;
};


// private header. Please don't look
/// @cond Hidden
class HardwareAccessHidden {
public:
	HardwareAccessHidden();
	~HardwareAccessHidden();

	PowerDistributionPanel panel;

	CachedSolenoid solenoidShifter;
	CachedSolenoid solenoidDrop;
	CachedSolenoid solenoidStabilizer;
	CachedSolenoid solenoidBrake;
	CachedSolenoid solenoidIntake;

	CachedTalon motorLeftFront;
	CachedTalon motorLeftBack;
	CachedTalon motorRightFront;
	CachedTalon motorRightBack;
	CachedTalon motorCenter;
	CachedTalon motorLiftLeft;
	CachedTalon motorLiftRight;
	CachedTalon motorGripper;
	CachedTalon motorIntakeLeft;
	CachedTalon motorIntakeRight;
	CachedTalon motorRCPuller;

	AnalogInput analogInGyroRate;

	DigitalInput diDriveEncoderLeftA;
	DigitalInput diDriveEncoderLeftB;
	DigitalInput diDriveEncoderRightA;
	DigitalInput diDriveEncoderRightB;
	DigitalInput diLifterEncoderA;
	DigitalInput diLifterEncoderB;
	DigitalInput diLifterFlagBottom;
	DigitalInput diGripperEncoderA;
	DigitalInput diGripperEncoderB;
	DigitalInput diGripperFlagA;

	DigitalInput diGripperBannerDetection;
	DigitalInput diLifterFlagTop;
	DigitalInput diGripperFlagB;

	BuiltInAccelerometer rawAccelerometer;

#if ONE_CHANNEL_DRIVE
	BidirectionalCounter encoderDriveLeft;
	BidirectionalCounter encoderDriveRight;
#else
	Encoder encoderDriveLeft;
	Encoder encoderDriveRight;
#endif
	Encoder encoderLifter;
	Encoder encoderGripper;

	ADW22307 driveGyro;
	IntegratedAccelerometer driveAccelerometer;
};
/// @endcond
