#include "lights.h"
#include "controls.h"
#include "util/common.h"

Lights::Lights(Drive& d, Gripper& g, Lifter& l, Controls& c) :
				SystemBase("Lights"),
				drive(d),
				gripper(g),
				lifter(l),
				controls(c),
				serialWriter("Lights Writer Thread",
						(FUNCPTR) Lights::callWriteToStream),
				Serial1(9600, SerialPort::kMXP) {
	serialWriter.Start((uint32_t) this);
}
Lights::~Lights() {

}
void Lights::debug(Logger& logger) {

}
void Lights::initialize() {
	Serial1.Reset();
}
void Lights::callWriteToStream(uint32_t arg) {
	((Lights*) arg)->writeToStream();
}

void Lights::writeToStream() {
	while (true) {
		char bytes[2];
		{
			Synchronized sync(semaphore);
			bytes[0] = output[0];
			bytes[1] = output[1];
		}
		printf("%x %x\n", bytes[0], bytes[1]);
		Serial1.Write(bytes, 2);
		Serial1.Flush();
		Wait(0.050);
	}
}

void Lights::process() {
	double lift = lifter.getPosition();
	int LiftLevel = lround(lift * 6);
	if (LiftLevel < 0)
		LiftLevel = 0;
	if (LiftLevel > 6)
		LiftLevel = 6;

	double Grip = gripper.getPosition();
	int GripPos = lrint(Grip * 8);
	if (GripPos < 0) {
		GripPos = 0;
	}
	if (GripPos > 7) {
		GripPos = 7;
	}

	bool WheelDown = drive.isDropWheelDown();
	//bool ShiftLow = drive.isShiftedDown();
	bool PongModeG = controls.PongMode();
	bool SignalLight = controls.shouldSignalWithLights();
	bool RightplayerForward = controls.rightPlayerForward();
	bool RightplayerBackwards = controls.rightPlayerBackwards();
	bool LeftPlayerForward = controls.leftPlayerForward();
	bool LeftPlayerBackwards = controls.leftPlayerBackwards();
	bool TeleopEnabled = RobotState.IsOperatorControl();

	char Bytes[2];
	Bytes[0] = 0;
	Bytes[1] = 1;

	if (WheelDown) {
		Bytes[0] = Bytes[0] | 0x02;
	}
	LiftLevel = LiftLevel << 2;
	Bytes[0] = Bytes[0] | (char) LiftLevel;

	GripPos = GripPos << 5;
	Bytes[0] = Bytes[0] | (char) GripPos;

	if (TeleopEnabled) {
		Bytes[1] = Bytes[1] | 0x02;
	}

	if (PongModeG) {
		Bytes[1] = Bytes[1] | 0x04;
	}

	if (SignalLight) {
		Bytes[1] = Bytes[1] | 0x80;
	}

	if (RightplayerForward) {
		Bytes[1] = Bytes[1] | 0x08;
	}

	if (RightplayerBackwards) {
		Bytes[1] = Bytes[1] | 0x10;
	}

	if (LeftPlayerForward) {
		Bytes[1] = Bytes[1] | 0x20;
	}

	if (LeftPlayerBackwards) {
		Bytes[1] = Bytes[1] | 0x40;
	}

	{
		Synchronized sync(semaphore);
		output[0] = Bytes[0];
		output[1] = Bytes[1];
	}
}
void Lights::blinkMode(bool blinking) {

}
