#include "logging.h"
#include <stdarg.h>
#include <fstream>
#include <algorithm>
#include <sstream>
#include <set>
#include "util/common.h"

const std::string FILE_PATH = "/media/sda1/";
const std::string DESTINATION = std::string(FILE_PATH).append("log.txt");

const double LOGGING_PERIOD = 0.130;

#define LOG_TO_FILE false

static std::set<SystemBase*> systemRegistry;

void Logging::registerSystem(SystemBase* p) {
	systemRegistry.insert(p);
}
void Logging::unregisterSystem(SystemBase* p) {
	systemRegistry.erase(p);
}

Logging::Logging() :
				SystemBase("Logging"),
				udpOutput(1140) {
	writingToDashboard = false;
	lastLogTime = 0;
}

Logging::~Logging() {

}

void Logging::initialize() {

}

bool compareSystems(const SystemBase* l, const SystemBase* r) {
	return l->getName() < r->getName();
}

#define NTTABLE_FALLBACK false

void Logging::process() {
	bool send;
	{
		double nextTime = GetTime();
		if (nextTime > lastLogTime + LOGGING_PERIOD) {
			lastLogTime = nextTime;
			send = true;
		} else {
			send = false;
		}
	}

	// NOTE: although std::string is horribly inefficient, replacing
	// it with fast buffer writes saves only a few 1e-5 per loop

	Logger logger;
	std::vector<SystemBase*> systems(systemRegistry.begin(), systemRegistry.end());
	std::sort(systems.begin(), systems.end(), compareSystems);
	for (SystemBase* system : systems) {
		system->debug(logger);
	}
	std::stringstream s;
	for (auto text : bundle) {
		s << text << "\n";
	}
	s << logger.getText();
	bundle.clear();
	std::string message = s.str();
	// TODO remove || true to let the driver control the logging state
	if (writingToDashboard || true) {
		if (send) {
			udpOutput.log(message.c_str(), message.size());
#if LOG_TO_FILE
	{
		std::ofstream logfile(DESTINATION, std::ios::app | std::ios::out);
		logfile << message;
	}
#endif
		}
	}
}

void Logging::setWritingToDashboard(bool pleaseDo) {
	writingToDashboard = pleaseDo;
}

void Logging::periodic(const char* fmt, ...) {
	char buffer[50];
	va_list args;
	va_start(args, fmt);
	vsnprintf(buffer, 50, fmt, args);
	va_end(args);
	bundle.push_back(buffer);
}
