#pragma once

#include "SampleRobot.h"
#include "systembase.h"
#include "drive.h"
#include "gripper.h"
#include "lifter.h"
#include "lights.h"
#include "vision.h"
#include "auto.h"
#include "controls.h"
#include "logging.h"
#include "rcpuller.h"
#include "hardware.h"

/**
 * @class Minesweeper
 * @brief This is the main robot class
 *
 * It's called Minesweeper, because it's called Minesweeper.
 * It organizes the various subsystems, delegating the actual
 * control and implementation work to them.
 *
 * @see Drive
 * @see Gripper
 * @see Lifter
 * @see AutoRunner
 * @see Controls
 */
class Minesweeper: public SampleRobot {
public:
	Minesweeper();
private:
	Drive drive;
	Gripper gripper;
	Lifter lifter;
	Logging logging;
	Vision vision;
	TinCanPuller puller;
	AutoRunner auton;
	Controls controls;
	Lights lights;

	// Order in which the systems will be run for autonomous and for
	// disabled/teleop modes
	std::vector<SystemBase*> autonRunOrder = {
			HardwareAccess::getInstance().getSensorSystem(),
			&auton,
			&puller,
			&gripper,
			&lifter,
			&drive,
			&lights,
			&logging,
			HardwareAccess::getInstance().getMotorSystem() };
	std::vector<SystemBase*> mainRunOrder = {
			HardwareAccess::getInstance().getSensorSystem(),
			&controls,
			&puller,
			&gripper,
			&lifter,
			&drive,
			&lights,
			&logging,
			HardwareAccess::getInstance().getMotorSystem() };

	void Autonomous();
	void Disabled();
	void OperatorControl();
	void MainInit();
	void Test();
	void RunPeriodic(const std::vector<SystemBase*>&);
	void Prestart();
	void RobotInit();
};
