#pragma once

#include "NIIMAQdx.h"
#include <string>

class Camera {
public:
	enum class Size {
		x1280y800,
		x1280y720,
		x960y544,
		x800y600,
		x800y448,
		x640y480,
		x640y360,
		x424y240,
		x352y288,
		x320y240,
		x176y144,
		x160y120
	};
	enum class FPS {
		fps30_00, fps20_00, fps15_00, fps10_00, fps7_50
	};
	enum class Type {
		YUY2, JPEG
	};
	Camera(const char* namex, Size s, FPS f = FPS::fps20_00,
			Type t = Type::JPEG);
	~Camera();
	Image* GetNewImage();
	/// @return Actual image size in buffer
	uint32_t GetNewImageData(uint8_t* buffer, uint32_t size);
private:
	IMAQdxSession s_id;
	IMAQdxError imaqError;
	Image * image;
	const char* name;
	bool isReady;
	double lastAttemptTime;
	std::string videoMode;

	bool PreCapture();
	void PostCapture(IMAQdxError e, uInt32 frameNo);
	void Fail();
	std::string ConstructMode(Size s, FPS f, Type t = Type::JPEG);
	void ConfigCameraMode();
	void Start();
	void Stop();
	static Camera* activeCamera;
};

