#pragma once

#include <stdint.h>
#include <Task.h>
#include <condition_variable>
#include <mutex>
#include <set>
#include "nivision.h"

class ImageServer {
public:
	static constexpr uint16_t kDefaultPort = 1180;
	typedef void (*DeleterFunc)(uint8_t*);
	ImageServer(uint16_t port = kDefaultPort);
	~ImageServer();
	void SetImage(Image* image);
	void SetImageRawData(uint8_t* data, uint32_t bufferSize, uint32_t imageSize,
			DeleterFunc deleter);
private:
	Task m_serverThread;
	Task m_connectionThread;
	typedef struct {
		uint8_t* data;
		uint32_t start;
		uint32_t imageSize;
		DeleterFunc deleter;
	} MemoryBuffer;
	MemoryBuffer m_imageData;
	std::condition_variable_any m_newImageVariable;
	std::recursive_mutex m_imageMutex;
	std::recursive_mutex m_connectionMutex;
	std::set<int> m_connections;
	uint16_t m_port;

	void FreeImageData(MemoryBuffer* imageData);
	static void IMAQDelete(uint8_t* data);
	void ManageConnections();
	void Serve();
	void FlushConnection(int conn);

	static void RunServe(uint32_t d0);
	static void RunManageConnections(uint32_t d1);
};
