#include "misc.h"
#include "Timer.h"
#include "common.h"
#include "HAL/HAL.hpp"
#include <sstream>
#include <string>
#include <vector>

CurrentMode_Signal RobotState;

void CurrentMode_Signal::updateMe() {
	HALControlWord control;
	HALGetControlWord(&control);
	enabled = control.enabled;
	autonomous = control.autonomous;
	test = control.test;
	eStop = control.eStop;
	fmsAttached = control.fmsAttached;
	dsAttached = control.dsAttached;
}
bool CurrentMode_Signal::IsEnabled() {
	return enabled;
}
bool CurrentMode_Signal::IsDisabled() {
	return !enabled;
}
bool CurrentMode_Signal::IsAutonomous() {
	return autonomous;
}
bool CurrentMode_Signal::IsTest() {
	return test;
}
bool CurrentMode_Signal::IsFMSConnected() {
	return fmsAttached;
}
bool CurrentMode_Signal::IsOperatorControl() {
	return !(test || autonomous);
}

double scaleLinear(double v, double old_min, double old_max, double new_min,
		double new_max) {
	return (v - old_min) / (old_max - old_min) * (new_max - new_min) + new_min;
}

double bound(double v, double min_or_max, double max_or_min) {
	double min, max;
	if (min_or_max > max_or_min) {
		max = min_or_max;
		min = max_or_min;
	} else {
		max = max_or_min;
		min = min_or_max;
	}

	if (v < min) {
		return min;
	}
	if (v > max) {
		return max;
	}
	return v;
}

double sign(double value) {
	if (value >= 0) {
		return 1.0;
	}
	return -1.0;
}

char Yn(bool Yes) {
	return Yes ? 'Y' : 'n';
}

std::vector<double> differentiate(const std::vector<double>& input) {
	std::vector<double> v;
	v.reserve(input.size() - 1);
	for (int i = 0; i < (int) input.size() - 1; i++) {
		v.push_back(input[i + 1] - input[i]);
	}
	return v;
}

TimerW::TimerW() {
	p = new Timer();
	p->Start();
}
TimerW::~TimerW() {
	delete p;
}
void TimerW::Start() {

}
double TimerW::Get() {
	return p->Get();

}
void TimerW::Reset() {
	p->Reset();
}

void normalizeBound2(double*a, double*b) {
	double sa = fabs(*a);
	double sb = fabs(*b);
	double max = sa;
	if (sb > max) {
		max = sb;
	}
	if (max < 1) {
		max = 1;
	}
	*a /= max;
	*b /= max;
}

void normalizeBound3(double*a, double*b, double*c) {
	double sa = fabs(*a);
	double sb = fabs(*b);
	double sc = fabs(*c);
	double max = sa;
	if (sb > max) {
		max = sb;
	}
	if (sc > max) {
		max = sc;
	}
	if (max < 1) {
		max = 1;
	}
	*a /= max;
	*b /= max;
	*c /= max;
}

struct ProfData {
	std::vector<double> times;
	std::vector<std::string> labels;
	bool running;
};

Profiler::Profiler() {
	p= new ProfData();
	p->running = false;
	start();
}
Profiler::~Profiler() {
	finish();
	delete p;
}
void Profiler::start() {
	if (p->running) {
		return;
	}
	p->running = true;
	p->times.push_back(GetTime());
}
void Profiler::add(const char* s) {
	if (!p->running) {
		start();
	}
	p->running = true;
	p->times.push_back(GetTime());
	p->labels.push_back(s);
}
void Profiler::add() {
	std::string s = std::to_string(p->times.size());
	add(s.c_str());
}
void Profiler::finish() {
	if (!p->running) {
		return;
	}
	std::stringstream s;
	for (uint32_t i = 1; i < p->times.size(); i++) {
		s << "{" << p->labels[i - 1] << " " << p->times[i] - p->times[i - 1] << " "
				<< p->times[i] - p->times[0] << " } ";
	}
	s << std::endl;
	std::cout << s.str();
	wipe();
}
void Profiler::wipe() {
	p->running = false;
	p->times.clear();
	p->labels.clear();
}
