#pragma once

/**
 * @brief Container for position data
 *
 * Combination of the core position data for an object on a 2d plane.
 * Relative to an arbitrary origin, an object has 2 dimensions of
 * translation and 1 of rotation.
 */
typedef struct {
	/// @brief X coordinate (left is negative, right is positive). Units are centimeters.
	double xPosition;
	/// @brief Y coordinate (up or forward is positive, down or back is negative). Units are centimeters.
	double yPosition;
	/// @brief Rotational coordinate (positive values for clockwise rotation). Units are degrees.
	double rotation;
} Position;


/**
 * Please only use this helper in the main thread. Thank you!
 */
class CurrentMode_Signal {
public:
	void updateMe();
	bool IsEnabled();
	bool IsAutonomous();
	bool IsTest();
	bool IsFMSConnected();
	bool IsOperatorControl();
	bool IsDisabled();
private:
	bool enabled=false;
	bool autonomous=false;
	bool test=false;
	bool eStop=false;
	bool fmsAttached=false;
	bool dsAttached=false;
};

extern CurrentMode_Signal RobotState;


double scaleLinear(double value, double old_min, double old_max, double new_min,
		double new_max);

/**
 * @brief Constrain a value to be within an interval
 *
 * @param value Input value
 * @param min Minimum value
 * @param max Maxinum possible value
 */
double bound(double value, double min_or_max, double max_or_min);

double sign(double value);

char Yn(bool Yes);

class Timer;
class TimerW {
public:
	TimerW();
	~TimerW();
	void Start();// no-op
	double Get();
	void Reset();
private:
	Timer* p;
};

void normalizeBound2(double*, double*);
void normalizeBound3(double*, double*, double*);

class Profiler {
public:
	Profiler();
	~Profiler();
	void start();
	void add(const char* s);
	void add();
	void finish();
	void wipe();
private:
	struct ProfData* p;
};
