#include "udplog.h"

#include <ifaddrs.h>
#include <list>
#include <net/if.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "util/common.h"
#include "HAL/HAL.hpp"
#include <netinet/in.h>
#include <HAL/Semaphore.hpp>
#include <string>

const double UPDATE_PERIOD = 0.100;

std::list<in_addr> getBroadcastAddrs() {
	static SEMAPHORE_ID semaphore = initializeSemaphore(1);
	static double lastCallTime = 0;
	static std::list<in_addr> items;

	{
		Synchronized sync(semaphore);

		if (GetTime() - lastCallTime < 0.100) {
			return items;
		}
		lastCallTime = GetTime();

		items.clear();
		struct ifaddrs* addrs;
		getifaddrs(&addrs);
		for (struct ifaddrs* ifa = addrs; ifa != NULL; ifa = ifa->ifa_next) {
			if (ifa->ifa_addr == 0) {
				continue;
			}
			if (ifa->ifa_addr->sa_family != AF_INET) {
				continue;
			}
			if (ifa->ifa_flags & IFF_BROADCAST) {
				sockaddr_in* addr = (sockaddr_in*) ifa->ifa_ifu.ifu_broadaddr;
				items.push_back(addr->sin_addr);
			}
		}
		freeifaddrs(addrs);
		return items;
	}
}

class UDPLog_p {
public:
	UDPLog_p(uint16_t port) {
		port_n = htons(port);

		sock_fd = socket(AF_INET, SOCK_DGRAM | SOCK_NONBLOCK, 0);
		int optval = 1;
		setsockopt(sock_fd, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval));
		if (sock_fd == -1) {
			printf("UDPLogger: Bad socket\n");
		}
	}
	~UDPLog_p() {
		shutdown(sock_fd, SHUT_RDWR);
		close(sock_fd);
	}

	void log(const char* text, size_t length) {
		if (sock_fd == -1) {
			return;
		}
		for (in_addr saddr : getBroadcastAddrs()) {
			sockaddr_in sa;
			sa.sin_port = port_n;
			sa.sin_family = AF_INET;
			sa.sin_addr = saddr;
			sendto(sock_fd, text, length, 0, (sockaddr*) &sa, sizeof(sa));
		}
	}
private:
	int sock_fd = 0;
	in_port_t port_n;
};

UDPLog::UDPLog(int port) {
	priv = new UDPLog_p((uint16_t) port);
}
UDPLog::~UDPLog() {
	delete priv;
}

void UDPLog::log(const char* s, size_t x) {
	priv->log(s, x);
}

void UDPLog::log(const char* fmt, ...) {
	char* buf = nullptr;
	va_list args;
	va_start(args, fmt);
	size_t len = vasprintf(&buf, fmt, args);
	va_end(args);

	priv->log(buf, len);
	free(buf);
}

