#include "vision.h"

#include "util/camera.h"
#include "util/imageserver.h"
#include "util/common.h"

#include "Task.h"

void warn(const char* s) {
	static double time = 0;
	double tt = GetTime();
	if (tt > time + 15.00) {
		time = tt;
		puts(s);
	}
}

class VisionCtr {
private:
	Task task;
	bool alt;
public:
	VisionCtr() :
					task("Vision Relay", (FUNCPTR) VisionCtr::callProcess) {
		alt = false;
		task.Start((uint32_t) this);
	}
	~VisionCtr() {
	}
	void sendImgWithCamera(Camera& camera, ImageServer &server) {
		uint32_t bufferSize = 1000000;
		uint8_t* buffer = new uint8_t[bufferSize];
		uint32_t imageSize;

		imageSize = camera.GetNewImageData(buffer, bufferSize);
		if (imageSize == 0) {
			warn("Image size is zero");
			delete buffer;
			return;
		}
		server.SetImageRawData(buffer, bufferSize, imageSize,
				&VisionCtr::NewDeleter);
	}
	static void NewDeleter(uint8_t* data) {
		delete[] data;
	}
	void run() {
		// TODO: some Size options are wrong, only for JPEG, some only for YUY2 (which we should never/rarely use)

		// You may wonder why the code cannot read from two cameras simultaneously
		// when they have low bandwidth requirements (ex. x160y120 at fps10_00).
		// This issue comes from the linux uvcvideo driver. To fix it, you must
		// set the option quirks=128 for the driver to reduce bandwidth limits.
		// This can be done permanently (taking effect on reboot) by creating
		// the file on the roborio /etc/modprobe.d/uvcvideo.conf
		// One way to do this is to ssh into the Roborio and run a short command.

		// In a good terminal (NOT CMD, Git Bash works, as should PuTTY/cygwin):
		//
		// $ ssh admin@roborio-1511.local
		// ... (password is blank) ...
		// admin@roboRIO-1511:~# echo "options uvcvideo quirks=128" > /etc/modprobe.d/uvcvideo.conf
		// admin@roboRIO-1511:~# reboot

		// Available ports:
		// 1180 TCP
		// 5800-5810 TCP/UDP
		// 1140 UDP Robot to Dashboard
		// 1130 UDP Dashboard to Robot
		ImageServer server0(1180);
		ImageServer server1(5800);
		Camera camera1("cam1", Camera::Size::x160y120, Camera::FPS::fps10_00);
		Camera camera0("cam0", Camera::Size::x160y120, Camera::FPS::fps10_00);
		while (true) {
			bool altx = alt;
			if (altx) {
				sendImgWithCamera(camera1, server1);
			}
			sendImgWithCamera(camera0, server0);
		}
	}
	static int callProcess(uint32_t a) {
		((VisionCtr*) a)->run();
		return 0;
	}
	void setAlternateMode(bool off) {
		alt = !off;
	}
}
;

Vision::Vision() :
				SystemBase("Vision") {
	p = new VisionCtr();
}

Vision::~Vision() {
	delete p;
}
void Vision::initialize() {
}
void Vision::process() {
}
void Vision::debug(Logger&) {
}
void Vision::setAlternateMode(bool off) {
	p->setAlternateMode(off);
}
